package CatalystX::CMS::tt;
use strict;

=head1 NAME

CatalystX::CMS::tt - base tt class 
 
=head1 DESCRIPTION

CatalystX::CMS::tt exists mainly so that Catalyst::Plugin::Static::Simple::ByClass
can easily locate the static and .tt files for serving in development.
 
=head1 METHODS

No methods in this class.

=cut

1;

__END__

=head1 AUTHOR

Peter Karman, C<< <karman@cpan.org> >>

=head1 BUGS

Please report any bugs or feature requests to
C<bug-catalystx-cms@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.  I will be notified, and then you'll automatically be
notified of progress on your bug as I make changes.

=head1 ACKNOWLEDGEMENTS

The Minnesota Supercomputing Institute C<< http://www.msi.umn.edu/ >>
sponsored the development of this software.

=head1 COPYRIGHT & LICENSE

Copyright 2008 by the Regents of the University of Minnesota.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

